/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 France Telecom R&D
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbsMDBTransportSingletonBean.java 20430 2010-10-07 12:49:57Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.cxf.transport.mdb;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;

import org.apache.cxf.Bus;
import org.apache.cxf.bus.spring.BusApplicationContext;
import org.apache.cxf.common.logging.LogUtils;
import org.apache.cxf.transport.Destination;
import org.springframework.context.ApplicationContext;

/**
 */
public abstract class AbsMDBTransportSingletonBean implements IMDBTransportSingletonBean {

    /**
     * Logger.
     */
    private static final Logger LOG = LogUtils.getL7dLogger(AbsMDBTransportSingletonBean.class);

    private MDBTransportFactory mdbTransportFactory = null;

    private String path = null;
    
    public AbsMDBTransportSingletonBean(String path) {
        this.path = path;
    }

    @PostConstruct
    public void init() {
        Enumeration<URL> enumURLs;
        try {
            enumURLs = getClass().getClassLoader().getResources(path);
        } catch (IOException e) {
            throw new IllegalStateException("Unable to find resources named '" + path + "' by using classloader '"
                    + getClass().getClassLoader() + "' of the class '" + getClass() + "'.", e);
        }

        if (enumURLs == null || !enumURLs.hasMoreElements()) {
            throw new IllegalStateException("No resource '" + path + "' have been found by using classloader '"
                    + getClass().getClassLoader() + "' of the class '" + getClass() + "'.");
        }

        // init endpoint
        List<URL> urls = new ArrayList<URL>();
        while (enumURLs.hasMoreElements()) {
            urls.add(enumURLs.nextElement());
        }
        ApplicationContext context = initBus(urls.toArray(new URL[urls.size()]));

        // Gets the transport factory that has been initialized
        this.mdbTransportFactory = (MDBTransportFactory) context.getBean(MDBTransportFactory.class.getName());

    }

    protected ApplicationContext initBus(URL[] urls) {

        // Remove stack trace
        // PrintStream previousErrorStream = System.err;
        // ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // PrintStream tempStream = new PrintStream(baos);
        // System.setErr(tempStream);
        // try {
        // Force initialization of the singleton Bus shared by all clients.
        return new BusApplicationContext(urls, true);
        // } finally {
        // System.setErr(previousErrorStream);
        // Check if there was an UnknownHostException
        // String errorMessage = baos.toString();
        // if (errorMessage != null && errorMessage.contains("java.net.UnknownHostException")) {
        // // debug level
        // LOG.fine("Error while initializing the Bus : '" + errorMessage + "'");
        // } else if (!"".equals(errorMessage)) {
        // // error level
        // LOG.severe("Error while initializing the Bus : '" + errorMessage + "'.");
        // }
        // }
    }

    public Destination getDestination(String queueName) {
        return mdbTransportFactory.getExistingDestination(queueName);
    }

    public Bus getBus() {
        return mdbTransportFactory.getBus();
    }
}
